@@ -1,11 +1,16 @@
+use std::cell::UnsafeCell;
 use std::ptr::{self, NonNull, addr_of_mut};
 use std::sync::atomic::Ordering::{Acquire, Relaxed, Release, SeqCst};
+#[cfg(not(feature = "loom"))]
+use std::sync::atomic::fence;
 use std::sync::atomic::{AtomicPtr, AtomicU8};
 
-use crate::collectible::{Collectible, Link};
-use crate::exit_guard::ExitGuard;
-use crate::maybe_std::fence as maybe_std_fence;
-use crate::{Epoch, Tag};
+#[cfg(feature = "loom")]
+use loom::sync::atomic::fence;
+
+use super::collectible::{Collectible, Link};
+use super::exit_guard::ExitGuard;
+use super::{Epoch, Tag};
 
 /// [`Collector`] is a garbage collector that reclaims thread-locally unreachable instances
 /// when they are globally unreachable.
@@ -36,19 +41,19 @@ struct CollectorAnchor;
 
 impl Collector {
     /// The number of quiescent states before an epoch update is triggered.
-    const CADENCE: u8 = u8::MAX;
+    const CADENCE: u8 = 1_u8 << 7;
 
     /// Represents a quiescent state.
-    const INACTIVE: u8 = 1_u8 << 6;
+    const INACTIVE: u8 = Epoch::NUM_EPOCHS;
 
     /// Represents a terminated thread state.
-    const INVALID: u8 = 1_u8 << 7;
+    const INVALID: u8 = Epoch::NUM_EPOCHS << 1;
 
     #[inline]
     /// Accelerates garbage collection.
-    pub(super) const fn accelerate(collector_ptr: *mut Collector) {
+    pub(super) const fn accelerate(collector_ptr: NonNull<Collector>) {
         unsafe {
-            (*collector_ptr).next_epoch_update = 0;
+            (*collector_ptr.as_ptr()).next_epoch_update = 0;
         }
     }
 
@@ -56,12 +61,14 @@ impl Collector {
     #[inline]
     pub(super) fn current() -> NonNull<Collector> {
         LOCAL_COLLECTOR.with(|local_collector| {
-            let mut collector_ptr = local_collector.load(Relaxed);
-            if collector_ptr.is_null() {
-                collector_ptr = COLLECTOR_ANCHOR.with(CollectorAnchor::alloc);
-                local_collector.store(collector_ptr, Relaxed);
+            let local_collector_ptr = local_collector.get();
+            unsafe {
+                NonNull::new(*local_collector_ptr).unwrap_or_else(|| {
+                    let collector_ptr = COLLECTOR_ANCHOR.with(CollectorAnchor::alloc);
+                    (*local_collector_ptr) = collector_ptr.as_ptr();
+                    collector_ptr
+                })
             }
-            unsafe { NonNull::new_unchecked(collector_ptr) }
         })
     }
 
@@ -71,25 +78,40 @@ impl Collector {
     ///
     /// The method may panic if the number of readers has reached `u32::MAX`.
     #[inline]
-    pub(super) fn new_guard(collector_ptr: *mut Collector) {
+    pub(super) fn new_guard(collector_ptr: NonNull<Collector>) {
         unsafe {
-            if (*collector_ptr).num_readers == 0 {
+            if (*collector_ptr.as_ptr()).num_readers == 0 {
                 debug_assert_eq!(
-                    (*collector_ptr).state.load(Relaxed) & Self::INACTIVE,
+                    (*collector_ptr.as_ptr()).state.load(Relaxed) & Self::INACTIVE,
                     Self::INACTIVE
                 );
-                (*collector_ptr).num_readers = 1;
+                (*collector_ptr.as_ptr()).num_readers = 1;
 
                 // The epoch value can be any number between the last time a guard was created in
                 // the thread and the most recent value of `GLOBAL_TOOR.epoch`.
                 let new_epoch = Epoch::from_u8(GLOBAL_ROOT.epoch.load(Relaxed));
+
+                // Every epoch update, pointer loading, memory retirement, and memory
+                // reclamation event is always placed between a pair `SeqCst` memory barrier events
+                // (one in this method, and the other one in `scan`) where those `SeqCst` memory
+                // barriers are globally ordered by definition. This property ensures that a retired
+                // memory region cannot be reclaimed until any threads holding a pointer to the
+                // region turn inactive, because, the reclaimer needs to wait for at least two
+                // `SeqCst` barrier events in `scan` to reclaim the memory region, and the fact that
+                // the other threads were able to load a valid pointer means that the thread was in
+                // between the same `SeqCst` barrier event pair or an older one; if the former, one
+                // of the two `scan` events must have observed that the thread was active (this
+                // cannot be achieved by `Release-Acquire` relationships), preventing the global
+                // epoch from advancing more than once; if the latter, trivial.
                 if cfg!(feature = "loom")
                     || cfg!(miri)
                     || cfg!(not(any(target_arch = "x86", target_arch = "x86_64")))
                 {
                     // What will happen after the fence strictly happens after the fence.
-                    (*collector_ptr).state.store(new_epoch.into(), Relaxed);
-                    maybe_std_fence(SeqCst);
+                    (*collector_ptr.as_ptr())
+                        .state
+                        .store(new_epoch.into(), Relaxed);
+                    fence(SeqCst);
                 } else {
                     // This special optimization is excerpted from
                     // [`crossbeam_epoch`](https://docs.rs/crossbeam-epoch/).
@@ -97,63 +119,65 @@ impl Collector {
                     // The rationale behind the code is, it compiles to `lock xchg` that
                     // practically acts as a full memory barrier on `X86`, and is much faster than
                     // `mfence`.
-                    (*collector_ptr).state.swap(new_epoch.into(), SeqCst);
+                    (*collector_ptr.as_ptr())
+                        .state
+                        .swap(new_epoch.into(), SeqCst);
                 }
-                if (*collector_ptr).announcement != new_epoch {
-                    (*collector_ptr).announcement = new_epoch;
-                    let mut exit_guard = ExitGuard::new(collector_ptr, |collector_ptr| {
-                        if !collector_ptr.is_null() {
-                            Self::end_guard(collector_ptr);
-                        }
+                if (*collector_ptr.as_ptr()).announcement != new_epoch {
+                    (*collector_ptr.as_ptr()).announcement = new_epoch;
+                    let exit_guard = ExitGuard::new((), |()| {
+                        Self::end_guard(collector_ptr);
                     });
-                    Collector::epoch_updated(*exit_guard);
-                    *exit_guard = ptr::null_mut();
+                    Collector::epoch_updated(collector_ptr);
+                    exit_guard.forget();
                 }
             } else {
-                debug_assert_eq!((*collector_ptr).state.load(Relaxed) & Self::INACTIVE, 0);
+                debug_assert_eq!(
+                    (*collector_ptr.as_ptr()).state.load(Relaxed) & Self::INACTIVE,
+                    0
+                );
                 assert_ne!(
-                    (*collector_ptr).num_readers,
+                    (*collector_ptr.as_ptr()).num_readers,
                     u32::MAX,
                     "Too many EBR guards"
                 );
-                (*collector_ptr).num_readers += 1;
+                (*collector_ptr.as_ptr()).num_readers += 1;
             }
         }
     }
 
     /// Acknowledges an existing [`Guard`](super::Guard) being dropped.
     #[inline]
-    pub(super) fn end_guard(collector_ptr: *mut Collector) {
+    pub(super) fn end_guard(collector_ptr: NonNull<Collector>) {
         unsafe {
-            debug_assert_eq!((*collector_ptr).state.load(Relaxed) & Self::INACTIVE, 0);
             debug_assert_eq!(
-                (*collector_ptr).state.load(Relaxed),
-                u8::from((*collector_ptr).announcement)
+                (*collector_ptr.as_ptr()).state.load(Relaxed) & Self::INACTIVE,
+                0
+            );
+            debug_assert_eq!(
+                (*collector_ptr.as_ptr()).state.load(Relaxed),
+                u8::from((*collector_ptr.as_ptr()).announcement)
             );
 
-            if (*collector_ptr).num_readers == 1 {
-                (*collector_ptr).num_readers = 0;
-                if (*collector_ptr).next_epoch_update == 0 {
+            if (*collector_ptr.as_ptr()).num_readers == 1 {
+                (*collector_ptr.as_ptr()).num_readers = 0;
+                if (*collector_ptr.as_ptr()).next_epoch_update == 0 {
                     Collector::scan(collector_ptr);
-                    (*collector_ptr).next_epoch_update = if (*collector_ptr).has_garbage {
-                        Self::CADENCE / 4
-                    } else {
-                        Self::CADENCE
-                    };
-                } else if (*collector_ptr).has_garbage
+                    (*collector_ptr.as_ptr()).next_epoch_update = Self::CADENCE;
+                } else if (*collector_ptr.as_ptr()).has_garbage
                     || Tag::into_tag(GLOBAL_ROOT.chain_head.load(Relaxed)) == Tag::Second
                 {
-                    (*collector_ptr).next_epoch_update -= 1;
+                    (*collector_ptr.as_ptr()).next_epoch_update -= 1;
                 }
 
-                // What has happened cannot be observed after the thread setting itself inactive has
-                // been witnessed.
-                (*collector_ptr).state.store(
-                    u8::from((*collector_ptr).announcement) | Self::INACTIVE,
+                // `Release` is needed to prevent any previous load operations in this thread from
+                // passing through.
+                (*collector_ptr.as_ptr()).state.store(
+                    u8::from((*collector_ptr.as_ptr()).announcement) | Self::INACTIVE,
                     Release,
                 );
             } else {
-                (*collector_ptr).num_readers -= 1;
+                (*collector_ptr.as_ptr()).num_readers -= 1;
             }
         }
     }
@@ -173,21 +197,27 @@ impl Collector {
         Epoch::from_u8(GLOBAL_ROOT.epoch.load(Relaxed))
     }
 
+    /// Returns `true` if the [`Collector`] has garbage.
+    #[inline]
+    pub(super) const fn has_garbage(collector_ptr: NonNull<Collector>) -> bool {
+        unsafe { (*collector_ptr.as_ptr()).has_garbage }
+    }
+
+    /// Sets the garbage flag to allow this thread to advance the global epoch.
+    #[inline]
+    pub(super) const fn set_has_garbage(collector_ptr: NonNull<Collector>) {
+        unsafe {
+            (*collector_ptr.as_ptr()).has_garbage = true;
+        }
+    }
+
     /// Collects garbage instances.
     #[inline]
-    pub(super) fn collect(collector_ptr: *mut Collector, instance_ptr: *mut dyn Collectible) {
-        if instance_ptr.is_null() {
-            return;
-        }
-
+    pub(super) fn collect(collector_ptr: NonNull<Collector>, instance_ptr: *mut dyn Collectible) {
         unsafe {
-            (*instance_ptr).set_next_ptr((*collector_ptr).current_instance_link.take());
-            (*collector_ptr).current_instance_link = NonNull::new(instance_ptr);
-            (*collector_ptr).next_epoch_update = (*collector_ptr)
-                .next_epoch_update
-                .saturating_sub(1)
-                .min(Self::CADENCE / 4);
-            (*collector_ptr).has_garbage = true;
+            (*instance_ptr).set_next_ptr((*collector_ptr.as_ptr()).current_instance_link.take());
+            (*collector_ptr.as_ptr()).current_instance_link = NonNull::new(instance_ptr);
+            (*collector_ptr.as_ptr()).has_garbage = true;
         }
     }
 
@@ -195,7 +225,8 @@ impl Collector {
     #[inline]
     pub(super) fn pass_garbage() -> bool {
         LOCAL_COLLECTOR.with(|local_collector| {
-            let collector_ptr = local_collector.load(Relaxed);
+            let local_collector_ptr = local_collector.get();
+            let collector_ptr = unsafe { *local_collector_ptr };
             if collector_ptr.is_null() {
                 return true;
             }
@@ -204,8 +235,12 @@ impl Collector {
                 return false;
             }
             if collector.has_garbage {
+                // In case the thread state is marked `Invalid`, a `Release` guard is required since
+                // any remaining garbage may be reclaimed by other threads.
                 collector.state.fetch_or(Collector::INVALID, Release);
-                local_collector.store(ptr::null_mut(), Relaxed);
+                unsafe {
+                    *local_collector_ptr = ptr::null_mut();
+                }
                 mark_scan_enforced();
             }
             true
@@ -213,10 +248,10 @@ impl Collector {
     }
 
     /// Allocates a new [`Collector`].
-    fn alloc() -> *mut Collector {
+    fn alloc() -> NonNull<Collector> {
         let mut boxed = Box::new(Collector::default());
         boxed.state.store(Self::INACTIVE, Relaxed);
-        boxed.next_epoch_update = u8::MAX;
+        boxed.next_epoch_update = Self::CADENCE;
 
         let ptr = Box::into_raw(boxed);
         let mut current = GLOBAL_ROOT.chain_head.load(Relaxed);
@@ -239,25 +274,29 @@ impl Collector {
                 break;
             }
         }
-        ptr
+        unsafe { NonNull::new_unchecked(ptr) }
     }
 
     /// Acknowledges a new global epoch.
-    fn epoch_updated(collector_ptr: *mut Collector) {
+    fn epoch_updated(collector_ptr: NonNull<Collector>) {
         unsafe {
-            debug_assert_eq!((*collector_ptr).state.load(Relaxed) & Self::INACTIVE, 0);
             debug_assert_eq!(
-                (*collector_ptr).state.load(Relaxed),
-                u8::from((*collector_ptr).announcement)
+                (*collector_ptr.as_ptr()).state.load(Relaxed) & Self::INACTIVE,
+                0
             );
-            if (*collector_ptr).has_garbage {
-                let mut garbage_link = (*collector_ptr).next_instance_link.take();
-                (*collector_ptr).next_instance_link =
-                    (*collector_ptr).previous_instance_link.take();
-                (*collector_ptr).previous_instance_link =
-                    (*collector_ptr).current_instance_link.take();
-                (*collector_ptr).has_garbage = (*collector_ptr).next_instance_link.is_some()
-                    || (*collector_ptr).previous_instance_link.is_some();
+            debug_assert_eq!(
+                (*collector_ptr.as_ptr()).state.load(Relaxed),
+                u8::from((*collector_ptr.as_ptr()).announcement)
+            );
+            if (*collector_ptr.as_ptr()).has_garbage {
+                let mut garbage_link = (*collector_ptr.as_ptr()).next_instance_link.take();
+                (*collector_ptr.as_ptr()).next_instance_link =
+                    (*collector_ptr.as_ptr()).previous_instance_link.take();
+                (*collector_ptr.as_ptr()).previous_instance_link =
+                    (*collector_ptr.as_ptr()).current_instance_link.take();
+                (*collector_ptr.as_ptr()).has_garbage =
+                    (*collector_ptr.as_ptr()).next_instance_link.is_some()
+                        || (*collector_ptr.as_ptr()).previous_instance_link.is_some();
                 while let Some(instance_ptr) = garbage_link.take() {
                     garbage_link = (*instance_ptr.as_ptr()).next_ptr();
                     let mut guard = ExitGuard::new(garbage_link, |mut garbage_link| {
@@ -277,13 +316,7 @@ impl Collector {
                     garbage_link = guard.take();
                 }
             }
-            if (*collector_ptr).has_garbage {
-                // The collector still has garbage instances.
-                (*collector_ptr).next_epoch_update = Self::CADENCE / 4;
-            } else {
-                // No need to update the epoch immediately.
-                (*collector_ptr).next_epoch_update = Self::CADENCE;
-            }
+            (*collector_ptr.as_ptr()).next_epoch_update = Self::CADENCE;
         }
     }
 
@@ -310,9 +343,17 @@ impl Collector {
     }
 
     /// Scans the [`Collector`] instances to update the global epoch.
-    fn scan(collector_ptr: *mut Collector) -> bool {
+    fn scan(collector_ptr: NonNull<Collector>) {
         unsafe {
-            debug_assert_eq!((*collector_ptr).state.load(Relaxed) & Self::INVALID, 0);
+            debug_assert_eq!(
+                (*collector_ptr.as_ptr()).state.load(Relaxed) & Self::INVALID,
+                0
+            );
+
+            if u8::from((*collector_ptr.as_ptr()).announcement) != GLOBAL_ROOT.epoch.load(Relaxed) {
+                // No need for further processing if the announcement is not up-to-date.
+                return;
+            }
 
             // Only one thread that acquires the chain lock is allowed to scan the thread-local
             // collectors.
@@ -320,18 +361,20 @@ impl Collector {
             if let Ok(mut current_collector_ptr) = lock_result {
                 let _guard = ExitGuard::new((), |()| Self::unlock_chain());
 
-                let known_epoch = (*collector_ptr).state.load(Relaxed);
+                let known_epoch = (*collector_ptr.as_ptr()).state.load(Relaxed);
                 let mut update_global_epoch = true;
                 let mut prev_collector_ptr: *mut Collector = ptr::null_mut();
                 while !current_collector_ptr.is_null() {
-                    if ptr::eq(collector_ptr, current_collector_ptr) {
+                    if ptr::eq(collector_ptr.as_ptr(), current_collector_ptr) {
                         prev_collector_ptr = current_collector_ptr;
-                        current_collector_ptr = (*collector_ptr).next_link.load(Relaxed);
+                        current_collector_ptr = (*collector_ptr.as_ptr()).next_link.load(Acquire);
                         continue;
                     }
 
+                    // `Acquire` is needed in case the other thread is inactive so that this thread
+                    // needs to reclaim memory for the thread.
                     let collector_state = (*current_collector_ptr).state.load(Acquire);
-                    let next_collector_ptr = (*current_collector_ptr).next_link.load(Relaxed);
+                    let next_collector_ptr = (*current_collector_ptr).next_link.load(Acquire);
                     if (collector_state & Self::INVALID) != 0 {
                         // The collector is obsolete.
                         let result = if prev_collector_ptr.is_null() {
@@ -350,7 +393,7 @@ impl Collector {
                         } else {
                             (*prev_collector_ptr)
                                 .next_link
-                                .store(next_collector_ptr, Relaxed);
+                                .store(next_collector_ptr, Release);
                             true
                         };
                         if result {
@@ -375,16 +418,13 @@ impl Collector {
                     // times of epoch updates. This `SeqCst` fence ensures that the epoch update is
                     // strictly sequenced after/before a `Guard`, enabling the event of the
                     // retirement of the memory region is also globally ordered with epoch updates.
-                    maybe_std_fence(SeqCst);
+                    fence(SeqCst);
                     GLOBAL_ROOT
                         .epoch
                         .store(Epoch::from_u8(known_epoch).next().into(), Relaxed);
-                    return true;
                 }
             }
         }
-
-        false
     }
 
     /// Clears the [`Collector`] chain to if all are invalid.
@@ -398,7 +438,7 @@ impl Collector {
                     if ((*current_collector_ptr).state.load(Acquire) & Self::INVALID) == 0 {
                         return false;
                     }
-                    current_collector_ptr = (*current_collector_ptr).next_link.load(Relaxed);
+                    current_collector_ptr = (*current_collector_ptr).next_link.load(Acquire);
                 }
 
                 // Reaching here means that there is no `Ptr` that possibly sees any garbage instances
@@ -416,7 +456,7 @@ impl Collector {
                 if result.is_ok() {
                     let mut current_collector_ptr = collector_head;
                     while !current_collector_ptr.is_null() {
-                        let next_collector_ptr = (*current_collector_ptr).next_link.load(Relaxed);
+                        let next_collector_ptr = (*current_collector_ptr).next_link.load(Acquire);
                         drop(Box::from_raw(current_collector_ptr));
                         current_collector_ptr = next_collector_ptr;
                     }
@@ -484,7 +524,7 @@ impl Collectible for Collector {
 }
 
 impl CollectorAnchor {
-    fn alloc(&self) -> *mut Collector {
+    fn alloc(&self) -> NonNull<Collector> {
         let _: &CollectorAnchor = self;
         Collector::alloc()
     }
@@ -496,20 +536,21 @@ impl Drop for CollectorAnchor {
         unsafe {
             // `LOCAL_COLLECTOR` is the last thread-local variable to be dropped.
             LOCAL_COLLECTOR.with(|local_collector| {
-                let collector_ptr = local_collector.load(Relaxed);
+                let local_collector_ptr = local_collector.get();
+                let collector_ptr = *local_collector_ptr;
                 if !collector_ptr.is_null() {
                     (*collector_ptr).state.fetch_or(Collector::INVALID, Release);
                 }
 
                 let mut temp_collector = Collector::default();
                 temp_collector.state.store(Collector::INACTIVE, Relaxed);
-                local_collector.store(addr_of_mut!(temp_collector), Release);
+                *local_collector_ptr = addr_of_mut!(temp_collector);
                 if !Collector::clear_chain() {
                     mark_scan_enforced();
                 }
 
                 Collector::clear_for_drop(addr_of_mut!(temp_collector));
-                local_collector.store(ptr::null_mut(), Release);
+                *local_collector_ptr = ptr::null_mut();
             });
         }
     }
@@ -530,7 +571,7 @@ fn mark_scan_enforced() {
 }
 
 thread_local! {
-    static LOCAL_COLLECTOR: AtomicPtr<Collector> = const { AtomicPtr::new(ptr::null_mut()) };
+    static LOCAL_COLLECTOR: UnsafeCell<*mut Collector> = const { UnsafeCell::new(ptr::null_mut()) };
     static COLLECTOR_ANCHOR: CollectorAnchor = const { CollectorAnchor };
 }
 
