@@ -1,15 +1,15 @@
 /// [`Epoch`] is a unit of time that dictates the lifetime of retired memory regions.
 ///
 /// The global epoch rotates `64` [`Epoch`] values in a range of `[0..63]`, instead of monotonically
-/// increasing to reduce the memory footprint.
+/// increasing to reduce the memory footprint of the [`Epoch`] values.
 #[derive(Clone, Copy, Debug, Default, Eq, Ord, PartialEq, PartialOrd)]
 pub struct Epoch {
     value: u8,
 }
 
 impl Epoch {
-    /// This crate uses `64` epoch values.
-    const NUM_EPOCHS: u8 = 64;
+    /// Rotates `64` epoch values.
+    pub(super) const NUM_EPOCHS: u8 = 64;
 
     /// Returns a future [`Epoch`] when the current readers will not be present.
     ///
@@ -112,13 +112,6 @@ impl Epoch {
     }
 }
 
-impl From<Epoch> for u8 {
-    #[inline]
-    fn from(epoch: Epoch) -> Self {
-        epoch.value
-    }
-}
-
 impl TryFrom<u8> for Epoch {
     type Error = Epoch;
 
@@ -133,3 +126,10 @@ impl TryFrom<u8> for Epoch {
         }
     }
 }
+
+impl From<Epoch> for u8 {
+    #[inline]
+    fn from(epoch: Epoch) -> Self {
+        epoch.value
+    }
+}
