@@ -1,7 +1,7 @@
 //! This module implements a simplified but safe version of
 //! [`scopeguard`](https://crates.io/crates/scopeguard).
 
-use std::mem::ManuallyDrop;
+use std::mem::{ManuallyDrop, forget};
 use std::ops::{Deref, DerefMut};
 
 /// [`ExitGuard`] captures the environment and invokes a defined closure at the end of the scope.
@@ -17,6 +17,15 @@ impl<T, F: FnOnce(T)> ExitGuard<T, F> {
             drop_callback: ManuallyDrop::new((captured, drop_callback)),
         }
     }
+
+    /// Forgets the [`ExitGuard`] without invoking the drop callback.
+    #[inline]
+    pub(crate) fn forget(mut self) {
+        unsafe {
+            ManuallyDrop::drop(&mut self.drop_callback);
+        }
+        forget(self);
+    }
 }
 
 impl<T, F: FnOnce(T)> Drop for ExitGuard<T, F> {
