@@ -1,9 +1,9 @@
 use std::panic::{RefUnwindSafe, UnwindSafe};
 use std::ptr::NonNull;
 
-use crate::Epoch;
-use crate::collectible::DeferredClosure;
-use crate::collector::Collector;
+use super::Epoch;
+use super::collectible::DeferredClosure;
+use super::collector::Collector;
 
 /// [`Guard`] allows the user to read [`AtomicShared`](super::AtomicShared) and keeps the
 /// underlying instance pinned to the thread.
@@ -35,7 +35,7 @@ impl Guard {
     #[must_use]
     pub fn new() -> Self {
         let collector_ptr = Collector::current();
-        Collector::new_guard(collector_ptr.as_ptr());
+        Collector::new_guard(collector_ptr);
         Self { collector_ptr }
     }
 
@@ -99,6 +99,44 @@ impl Guard {
         Collector::current_epoch()
     }
 
+    /// Returns `true` if the thread local container has garbage.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use sdd::{Guard, Shared};
+    ///
+    /// let guard = Guard::new();
+    ///
+    /// assert!(!guard.has_garbage());
+    ///
+    /// drop(Shared::new(1_usize));
+    /// assert!(guard.has_garbage());
+    /// ```
+    #[inline]
+    #[must_use]
+    pub const fn has_garbage(&self) -> bool {
+        Collector::has_garbage(self.collector_ptr)
+    }
+
+    /// Sets the garbage flag to allow this thread to advance the global epoch.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use sdd::Guard;
+    ///
+    /// let guard = Guard::new();
+    ///
+    /// assert!(!guard.has_garbage());
+    /// guard.set_has_garbage();
+    /// assert!(guard.has_garbage());
+    /// ```
+    #[inline]
+    pub const fn set_has_garbage(&self) {
+        Collector::set_has_garbage(self.collector_ptr);
+    }
+
     /// Forces the [`Guard`] to try to start a new epoch when it is dropped.
     ///
     /// # Examples
@@ -117,7 +155,7 @@ impl Guard {
     /// ```
     #[inline]
     pub const fn accelerate(&self) {
-        Collector::accelerate(self.collector_ptr.as_ptr());
+        Collector::accelerate(self.collector_ptr);
     }
 
     /// Executes the supplied closure at a later point of time.
@@ -137,7 +175,7 @@ impl Guard {
     #[inline]
     pub fn defer_execute<F: 'static + FnOnce()>(&self, f: F) {
         Collector::collect(
-            self.collector_ptr.as_ptr(),
+            self.collector_ptr,
             Box::into_raw(Box::new(DeferredClosure::new(f))),
         );
     }
@@ -153,7 +191,7 @@ impl Default for Guard {
 impl Drop for Guard {
     #[inline]
     fn drop(&mut self) {
-        Collector::end_guard(self.collector_ptr.as_ptr());
+        Collector::end_guard(self.collector_ptr);
     }
 }
 
