@@ -7,21 +7,33 @@ pub use atomic_owned::AtomicOwned;
 mod atomic_shared;
 pub use atomic_shared::AtomicShared;
 
+pub mod bag;
+pub use bag::Bag;
+
 mod epoch;
 pub use epoch::Epoch;
 
 mod guard;
 pub use guard::Guard;
 
+mod linked_list;
+pub use linked_list::{LinkedEntry, LinkedList};
+
 mod owned;
 pub use owned::Owned;
 
 mod ptr;
 pub use ptr::Ptr;
 
+pub mod queue;
+pub use queue::Queue;
+
 mod shared;
 pub use shared::Shared;
 
+pub mod stack;
+pub use stack::Stack;
+
 mod tag;
 pub use tag::Tag;
 
@@ -55,15 +67,5 @@ pub fn suspend() -> bool {
     collector::Collector::pass_garbage()
 }
 
-#[cfg(feature = "loom")]
-mod maybe_std {
-    pub(crate) use loom::sync::atomic::{AtomicPtr, fence};
-}
-
-#[cfg(not(feature = "loom"))]
-mod maybe_std {
-    pub(crate) use std::sync::atomic::{AtomicPtr, fence};
-}
-
 #[cfg(test)]
 mod tests;
