@@ -1,10 +1,11 @@
+use std::mem::offset_of;
 use std::ops::Deref;
 use std::ptr::{self, NonNull, addr_of};
 use std::sync::atomic::AtomicUsize;
 use std::sync::atomic::Ordering::{self, Relaxed};
 
-use crate::collectible::{Collectible, Link};
-use crate::collector::Collector;
+use super::collectible::{Collectible, Link};
+use super::collector::Collector;
 
 /// [`RefCounted`] stores an instance of type `T`, and a union of a link to the next
 /// [`Collectible`] or the reference counter.
@@ -98,7 +99,7 @@ impl<T> RefCounted<T> {
         let mut current = self.ref_cnt().load(Relaxed);
         loop {
             debug_assert_ne!(current, 0);
-            let new = if current <= 1 { 0 } else { current - 2 };
+            let new = current.saturating_sub(2);
             match self
                 .ref_cnt()
                 .compare_exchange_weak(current, new, Relaxed, Relaxed)
@@ -122,6 +123,13 @@ impl<T> RefCounted<T> {
         }
     }
 
+    /// Returns a non-null pointer to the instance.
+    #[inline]
+    pub(super) const fn inst_non_null_ptr(self_ptr: NonNull<Self>) -> NonNull<T> {
+        let offset = offset_of!(Self, instance);
+        unsafe { self_ptr.cast::<u8>().add(offset).cast::<T>() }
+    }
+
     /// Returns a reference to its reference count.
     #[inline]
     pub(super) fn ref_cnt(&self) -> &AtomicUsize {
@@ -131,7 +139,17 @@ impl<T> RefCounted<T> {
     /// Passes a pointer to [`RefCounted`] to the garbage collector.
     #[inline]
     pub(super) fn pass_to_collector(ptr: *mut Self) {
-        Collector::collect(Collector::current().as_ptr(), ptr as *mut dyn Collectible);
+        // The lifetime of the pointer is extended to `'static`; it is not illegal since the safe
+        // public API does not permit to create a `RefCounted` of a non `'static` type, and those
+        // that allow to do so are all unsafe methods (e.g., `{Owned, Shard}::new_unchecked`) where
+        // the doc explicitly states that the pointed-to value may be dropped at an arbitrary time.
+        #[allow(clippy::transmute_ptr_to_ptr)]
+        let ptr = unsafe {
+            std::mem::transmute::<*mut (dyn Collectible + '_), *mut (dyn Collectible + 'static)>(
+                ptr,
+            )
+        };
+        Collector::collect(Collector::current(), ptr);
     }
 }
 
